// D3DExample2.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
//
// Define version of Direct3D
#define DIRECT3D_VERSION 0x0800
//
// Include Direct3D headers
#include <d3d8.h>
#include <D3dx8core.h>

#define		WINDOWED		TRUE
#define		RES_X			1600
#define		RES_Y			1200
#define		DEPTH			32

HRESULT WinInit(HINSTANCE hInstance);
LRESULT CALLBACK MainWndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam );
void	UpdateFrame();
void	Clean();

HWND					g_hWnd;
IDirect3D8	*			g_lpD3DObject = NULL;
IDirect3DDevice8*		g_lpD3DDevice = NULL;
IDirect3DVertexBuffer8*	g_lpD3DVB = NULL;
IDirect3DIndexBuffer8*	g_lpD3DIB = NULL;

struct TLVERTEX {
	float	x, y, z, rhw;
	DWORD	dwColor;
};

#define	TLVERTEXFORMAT	D3DFVF_XYZRHW|D3DFVF_DIFFUSE 

TLVERTEX	arVertices[] = { 
	{200.0f, 200.0f, 0.5f, 1.0f, 0xFFFF0000}, 
	{600.0f, 200.0f, 0.5f, 1.0f, 0xFF0000FF},
	{200.0f, 600.0f, 0.5f, 1.0f, 0xFFFFFF00},
	{600.0f, 600.0f, 0.5f, 1.0f, 0xFF00FF00}};

WORD		arIndices[] = { 0, 1, 2, 1, 3, 2};


int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	DWORD dwRet;
 	// Window initialization
	dwRet = WinInit(hInstance);
	if(dwRet != ERROR_SUCCESS) {
		MessageBox(NULL, "Cannot create main window.", "Error", MB_ICONERROR);
		Clean();
		return 1;		
	}

	// D3D inicialization
	g_lpD3DObject = Direct3DCreate8(D3D_SDK_VERSION);
	if(!g_lpD3DObject) {		
		MessageBox(NULL, "Please, install DirectX 8.1.", "Error", MB_ICONERROR);
		Clean();
		return 1;
	}

	D3DPRESENT_PARAMETERS	d3dDeviceParam;
	// 
	// Set properties of device	
	ZeroMemory(&d3dDeviceParam, sizeof(d3dDeviceParam));
	d3dDeviceParam.Windowed							= WINDOWED;
	d3dDeviceParam.SwapEffect						= D3DSWAPEFFECT_DISCARD;
    d3dDeviceParam.BackBufferWidth					= RES_X;
    d3dDeviceParam.BackBufferHeight					= RES_Y;
    d3dDeviceParam.BackBufferFormat					= (DEPTH == 16) ? D3DFMT_R5G6B5 : D3DFMT_X8R8G8B8;
	d3dDeviceParam.BackBufferCount	 				= 1;	
	d3dDeviceParam.hDeviceWindow					= g_hWnd;
	d3dDeviceParam.MultiSampleType					= D3DMULTISAMPLE_NONE;

	if(WINDOWED) {
		D3DDISPLAYMODE displaymode;
		RECT rcWindowClient;
		//
		// Get window position
		GetClientRect(g_hWnd, &rcWindowClient);
		// Get current display mode
		g_lpD3DObject->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &displaymode);
		//
		// Set size of back buffer
        d3dDeviceParam.BackBufferWidth  = rcWindowClient.right - rcWindowClient.left;
        d3dDeviceParam.BackBufferHeight  = rcWindowClient.bottom - rcWindowClient.top;
		// Set display mode to current
        d3dDeviceParam.BackBufferFormat = displaymode.Format;
	}

	//
	// Create Device
	dwRet = g_lpD3DObject->CreateDevice(0, D3DDEVTYPE_HAL  , g_hWnd,
										 D3DCREATE_SOFTWARE_VERTEXPROCESSING,	
										 &d3dDeviceParam, &g_lpD3DDevice);
	if(dwRet != D3D_OK) {
		MessageBox(NULL, "Cannot create device.", "Error", MB_ICONERROR);
		Clean();
		return 2;
	}
	//
	// Create vertex buffer for our triangle
	g_lpD3DDevice->CreateVertexBuffer(sizeof(TLVERTEX) * 4, D3DUSAGE_WRITEONLY, TLVERTEXFORMAT, D3DPOOL_DEFAULT, &g_lpD3DVB);

	TLVERTEX	*pVertices;
	// Fill vertex buffer
	if(SUCCEEDED(g_lpD3DVB->Lock(0, 0, (BYTE**) &pVertices, 0))) {

		pVertices[0] = arVertices[0];
		pVertices[1] = arVertices[1];
		pVertices[2] = arVertices[2];
		pVertices[3] = arVertices[3];

		g_lpD3DVB->Unlock();
	}

	g_lpD3DDevice->CreateIndexBuffer(6 * sizeof(WORD), D3DUSAGE_WRITEONLY , D3DFMT_INDEX16 , D3DPOOL_DEFAULT, &g_lpD3DIB);

	WORD *pdwIndex;
	if(SUCCEEDED(g_lpD3DIB->Lock(0, 0, (BYTE**) &pdwIndex, 0))) {

		for(int i = 0; i < 6; i++) {
			pdwIndex[i] = arIndices[i];
		}
		g_lpD3DVB->Unlock();
	}
		

	MSG msg; 
	// Run message loop
	while( TRUE )
    {
        // Look for messages, if none are found then 
        // update the state and display it
        if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
        {
            if( 0 == GetMessage(&msg, NULL, 0, 0 ) )
            {
                // WM_QUIT was posted, so exit
				Clean();
                return (int)msg.wParam;
            }
			
            // Translate and dispatch the message
            TranslateMessage( &msg ); 
            DispatchMessage( &msg );
        }
        else
        {
			UpdateFrame();			
		}
    }		
	return 0;
}

void	Clean() {

	if(g_lpD3DDevice) {
		g_lpD3DDevice->Release();
	}

	if(g_lpD3DObject) {
		g_lpD3DObject->Release();
	}
	if(g_lpD3DVB) {
		g_lpD3DVB->Release();
	}
	if(g_lpD3DIB) {
		g_lpD3DIB->Release();
	}
}

HRESULT WinInit(HINSTANCE hInstance) 
{
	DWORD dwRet;
	WNDCLASSEX wc;		
	//
    // Register the Window Class
	wc.cbSize		 = sizeof(WNDCLASSEX);
    wc.lpszClassName = "D3DExample2Class";
    wc.lpfnWndProc   = MainWndProc;
    wc.style         = CS_VREDRAW|CS_HREDRAW;
    wc.hInstance     = hInstance;
    wc.hIcon         = NULL;
    wc.hCursor       = NULL;
    wc.hbrBackground = (HBRUSH) (COLOR_WINDOW + 1);
    wc.lpszMenuName  = NULL;   
	wc.hIconSm		 = NULL;
	wc.cbClsExtra    = 0;
	wc.cbWndExtra	 = 0;
	//
	// Register window class
    if(RegisterClassEx(&wc) == 0 ) {
		dwRet = GetLastError();		
		return dwRet;		
	}	
	//
	// Create new main window
	g_hWnd = CreateWindow("D3DExample2Class", "D3DExample2Window", 
							0, 0, 0, 1024, 768, NULL, NULL, hInstance, NULL);
	if(!g_hWnd) {
		dwRet = GetLastError();		
		return dwRet;
	}
	ShowWindow(g_hWnd, SW_SHOW);
	UpdateWindow(g_hWnd);
	dwRet = ERROR_SUCCESS;

	return dwRet;
}

LRESULT CALLBACK MainWndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
	switch(msg) {
	case WM_SETCURSOR:
		SetCursor(NULL);		
		break;
	
	case WM_DESTROY:
            // Cleanup and close the app            
            PostQuitMessage( 0 );
            return 0L;

	case WM_SYSCOMMAND:
        // Prevent moving/sizing and power loss in fullscreen mode
        switch( wParam )
        {
            case SC_MOVE:
            case SC_SIZE:
            case SC_MAXIMIZE:
            case SC_KEYMENU:
            case SC_MONITORPOWER:
                return 1;                
        }
        break;		
	}	
	return DefWindowProc(hWnd, msg, wParam, lParam);
}

void	UpdateFrame()
{	
	if(g_lpD3DDevice) {
		//
		// Clear the back buffer to a blue color
		g_lpD3DDevice->Clear(0, NULL, D3DCLEAR_TARGET, D3DCOLOR_XRGB(100, 100, 100), 0, 0);				
		//
		// Begin rendering
		if(SUCCEEDED(g_lpD3DDevice->BeginScene())) {
			// 
			// Set current vertex buffer
			g_lpD3DDevice->SetStreamSource(0, g_lpD3DVB, sizeof(TLVERTEX));
			g_lpD3DDevice->SetIndices(g_lpD3DIB, 0);
			// and type of vertex data
			g_lpD3DDevice->SetVertexShader(TLVERTEXFORMAT);
			//
			// Draw triangle		
			g_lpD3DDevice->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, 4, 0, 2);			
			g_lpD3DDevice->EndScene();
		}
		// Flip front/back buffers
		g_lpD3DDevice->Present(NULL, NULL, NULL, NULL);
	}
}